@@ -2,7 +2,7 @@
 #define SMP_LIB_H
 
 /*
- * Copyright (c) 2006-2011 Douglas Gilbert.
+ * Copyright (c) 2006-2013 Douglas Gilbert.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -117,6 +117,7 @@ extern "C" {
 #define SMP_FRES_SAVING_NOT_SUPPORTED 0x27
 #define SMP_FRES_SOURCE_ZONE_GROUP 0x28
 #define SMP_FRES_DIS_PASSWORD_NOT_SUPPORTED 0x29
+#define SMP_FRES_INVALID_FIELD_IN_REQUEST 0x2a
 
 /* Utilities can use these process status values for syntax errors and
    file (device node) problems (e.g. not found or permissions). Numbers
@@ -206,7 +207,7 @@ extern const char * smp_lib_version();
 
 struct smp_val_name {
     int value;
-    char * name;
+    const char * name;
 };
 
 /* <<< General purpose (i.e. not SMP specific) utility functions >>> */
