@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2006-2011 Douglas Gilbert.
+ * Copyright (c) 2006-2014 Douglas Gilbert.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -48,19 +48,21 @@
  *
  * This utility issues a DISCOVER function and outputs its response.
  *
- * Defined in SAS-2 (most recent draft sas2r16.pdf) and SPL which defines
- * the upper layers of SAS-2.1 . The most recent SPL draft is spl-r07.pdf .
+ * First defined in SAS-1. From and including SAS-2.1 this function is
+ * defined in the SPL series. The most recent SPL-3 draft is spl3r07.pdf .
  */
 
-static char * version_str = "1.34 20111221";    /* spl2r03 */
+static const char * version_str = "1.46 20140526";    /* spl3r07 */
 
 
 #define SMP_FN_DISCOVER_RESP_LEN 124
+#define SMP_FN_REPORT_GENERAL_RESP_LEN 76
 
 
 struct opts_t {
     int do_adn;
     int do_brief;
+    int do_cap_phy;
     int do_hex;
     int ign_zp;
     int do_list;
@@ -78,23 +80,24 @@ struct opts_t {
 };
 
 static struct option long_options[] = {
-        {"adn", 0, 0, 'A'},
-        {"brief", 0, 0, 'b'},
-        {"help", 0, 0, 'h'},
-        {"hex", 0, 0, 'H'},
-        {"ignore", 0, 0, 'i'},
-        {"interface", 1, 0, 'I'},
-        {"list", 0, 0, 'l'},
-        {"multiple", 0, 0, 'm'},
-        {"my", 0, 0, 'M'},
-        {"num", 1, 0, 'n'},
-        {"phy", 1, 0, 'p'},
-        {"sa", 1, 0, 's'},
-        {"summary", 0, 0, 'S'},
-        {"raw", 0, 0, 'r'},
-        {"verbose", 0, 0, 'v'},
-        {"version", 0, 0, 'V'},
-        {"zero", 0, 0, 'z'},
+        {"adn", no_argument, 0, 'A'},
+        {"brief", no_argument, 0, 'b'},
+        {"cap", no_argument, 0, 'c'},
+        {"help", no_argument, 0, 'h'},
+        {"hex", no_argument, 0, 'H'},
+        {"ignore", no_argument, 0, 'i'},
+        {"interface", required_argument, 0, 'I'},
+        {"list", no_argument, 0, 'l'},
+        {"multiple", no_argument, 0, 'm'},
+        {"my", no_argument, 0, 'M'},
+        {"num", required_argument, 0, 'n'},
+        {"phy", required_argument, 0, 'p'},
+        {"sa", required_argument, 0, 's'},
+        {"summary", no_argument, 0, 'S'},
+        {"raw", no_argument, 0, 'r'},
+        {"verbose", no_argument, 0, 'v'},
+        {"version", no_argument, 0, 'V'},
+        {"zero", no_argument, 0, 'z'},
         {0, 0, 0, 0},
 };
 
@@ -103,9 +106,10 @@ static void
 usage(void)
 {
     fprintf(stderr, "Usage: "
-          "smp_discover [--adn] [--brief] [--help] [--hex] [--ignore]\n"
-          "                    [--interface=PARAMS] [--list] [--my] "
-          "[--multiple]\n"
+          "smp_discover [--adn] [--brief] [--cap] [--help] [--hex] "
+          "[--ignore]\n"
+          "                    [--interface=PARAMS] [--list] "
+          "[--multiple] [--my]\n"
           "                    [--num=NUM] [--phy=ID] [--raw] "
           "[--sa=SAS_ADDR]\n"
           "                    [--summary] [--verbose] [--version] "
@@ -117,6 +121,7 @@ usage(void)
           "                         phy mode (i.e. with --multiple)\n"
           "    --brief|-b           less output, can be used multiple "
           "times\n"
+          "    --cap|-c             decode phy capabilities bits\n"
           "    --help|-h            print out usage message\n"
           "    --hex|-H             print response in hexadecimal\n"
           "    --ignore|-i          sets the Ignore Zone Group bit; "
@@ -127,6 +132,8 @@ usage(void)
           "    --list|-l            output attribute=value, 1 per line\n"
           "    --multiple|-m        query multiple phys, output 1 line "
           "for each\n"
+          "                         if given twice, full output for each "
+          "phy\n"
           "    --my|-M              output my (expander's) SAS address\n"
           "    --num=NUM|-n NUM     number of phys to fetch when '-m' "
           "is given\n"
@@ -166,9 +173,89 @@ dStrRaw(const char* str, int len)
         printf("%c", str[k]);
 }
 
-static char * smp_attached_device_type[] = {
+/* Returns 1 if 'Table to Table Supported' bit set on REPORT GENERAL
+ * respone. Returns 0 otherwise. */
+static int
+has_table2table_routing(struct smp_target_obj * top, const struct opts_t * op)
+{
+    unsigned char smp_req[] = {SMP_FRAME_TYPE_REQ, SMP_FN_REPORT_GENERAL,
+                               0, 0, 0, 0, 0, 0};
+    struct smp_req_resp smp_rr;
+    unsigned char rp[SMP_FN_REPORT_GENERAL_RESP_LEN];
+    char b[256];
+    char * cp;
+    int len, res, k, act_resplen;
+
+    memset(rp, 0, sizeof(rp));
+    if (op->verbose) {
+        fprintf(stderr, "    Report general request: ");
+        for (k = 0; k < (int)sizeof(smp_req); ++k)
+            fprintf(stderr, "%02x ", smp_req[k]);
+        fprintf(stderr, "\n");
+    }
+    memset(&smp_rr, 0, sizeof(smp_rr));
+    smp_rr.request_len = sizeof(smp_req);
+    smp_rr.request = smp_req;
+    smp_rr.max_response_len = sizeof(rp);
+    smp_rr.response = rp;
+    res = smp_send_req(top, &smp_rr, op->verbose);
+
+    if (res) {
+        fprintf(stderr, "RG smp_send_req failed, res=%d\n", res);
+        if (0 == op->verbose)
+            fprintf(stderr, "    try adding '-v' option for more debug\n");
+        return 0;
+    }
+    if (smp_rr.transport_err) {
+        fprintf(stderr, "RG smp_send_req transport_error=%d\n",
+                smp_rr.transport_err);
+        return 0;
+    }
+    act_resplen = smp_rr.act_response_len;
+    if ((act_resplen >= 0) && (act_resplen < 4)) {
+        fprintf(stderr, "RG response too short, len=%d\n", act_resplen);
+        return 0;
+    }
+    len = rp[3];
+    if ((0 == len) && (0 == rp[2])) {
+        len = smp_get_func_def_resp_len(rp[1]);
+        if (len < 0) {
+            len = 0;
+            if (op->verbose > 1)
+                fprintf(stderr, "unable to determine RG response length\n");
+        }
+    }
+    len = 4 + (len * 4);        /* length in bytes, excluding 4 byte CRC */
+    if ((act_resplen >= 0) && (len > act_resplen)) {
+        if (op->verbose)
+            fprintf(stderr, "actual RG response length [%d] less than "
+                    "deduced length [%d]\n", act_resplen, len);
+        len = act_resplen;
+    }
+    /* ignore --hex and --raw */
+    if (SMP_FRAME_TYPE_RESP != rp[0]) {
+        fprintf(stderr, "RG expected SMP frame response type, got=0x%x\n",
+                rp[0]);
+        return 0;
+    }
+    if (rp[1] != smp_req[1]) {
+        fprintf(stderr, "RG Expected function code=0x%x, got=0x%x\n",
+                smp_req[1], rp[1]);
+        return 0;
+    }
+    if (rp[2]) {
+        if (op->verbose > 1) {
+            cp = smp_get_func_res_str(rp[2], sizeof(b), b);
+            fprintf(stderr, "Report General result: %s\n", cp);
+        }
+        return 0;
+    }
+    return (len > 10) ? !!(0x80 & rp[10]) : 0;
+}
+
+static const char * smp_attached_device_type[] = {
     "no device attached",
-    "end device",
+    "SAS or SATA device",         /* was 'end device' */
     "expander device",            /* was 'edge expander' in SAS-1.1 */
     "expander device (fanout)",   /* marked as obsolete in SAS-2.0 */
     "reserved [4]",
@@ -177,7 +264,7 @@ static char * smp_attached_device_type[] = {
     "reserved [7]",
 };
 
-static char * smp_short_attached_device_type[] = {
+static const char * smp_short_attached_device_type[] = {
     "",         /* was "no " */
     "",         /* was "end" */
     "exp",
@@ -335,7 +422,7 @@ find_sas_connector_type(int conn_type, char * buff, int buff_len)
 static int
 do_discover(struct smp_target_obj * top, int disc_phy_id,
             unsigned char * resp, int max_resp_len,
-            int silence_err_report, const struct opts_t * optsp)
+            int silence_err_report, const struct opts_t * op)
 {
     unsigned char smp_req[] = {SMP_FRAME_TYPE_REQ, SMP_FN_DISCOVER, 0, 0,
                                0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
@@ -345,15 +432,15 @@ do_discover(struct smp_target_obj * top, int disc_phy_id,
     int len, res, k, act_resplen;
 
     memset(resp, 0, max_resp_len);
-    if (! optsp->do_zero) {     /* SAS-2 or later */
+    if (! op->do_zero) {     /* SAS-2 or later */
         len = (max_resp_len - 8) / 4;
         smp_req[2] = (len < 0x100) ? len : 0xff; /* Allocated Response Len */
         smp_req[3] = 2; /* Request Length: in dwords */
     }
-    if (optsp->ign_zp)
+    if (op->ign_zp)
         smp_req[8] |= 0x1;
     smp_req[9] = disc_phy_id;
-    if (optsp->verbose) {
+    if (op->verbose) {
         fprintf(stderr, "    Discover request: ");
         for (k = 0; k < (int)sizeof(smp_req); ++k)
             fprintf(stderr, "%02x ", smp_req[k]);
@@ -364,11 +451,11 @@ do_discover(struct smp_target_obj * top, int disc_phy_id,
     smp_rr.request = smp_req;
     smp_rr.max_response_len = max_resp_len;
     smp_rr.response = resp;
-    res = smp_send_req(top, &smp_rr, optsp->verbose);
+    res = smp_send_req(top, &smp_rr, op->verbose);
 
     if (res) {
         fprintf(stderr, "smp_send_req failed, res=%d\n", res);
-        if (0 == optsp->verbose)
+        if (0 == op->verbose)
             fprintf(stderr, "    try adding '-v' option for more debug\n");
         return -1;
     }
@@ -387,19 +474,19 @@ do_discover(struct smp_target_obj * top, int disc_phy_id,
         len = smp_get_func_def_resp_len(resp[1]);
         if (len < 0) {
             len = 0;
-            if (optsp->verbose > 1)
+            if (op->verbose > 1)
                 fprintf(stderr, "unable to determine response length\n");
         }
     }
     len = 4 + (len * 4);        /* length in bytes, excluding 4 byte CRC */
     if ((act_resplen >= 0) && (len > act_resplen)) {
-        if (optsp->verbose)
+        if (op->verbose)
             fprintf(stderr, "actual response length [%d] less than "
                     "deduced length [%d]\n", act_resplen, len);
-        len = act_resplen; 
+        len = act_resplen;
     }
-    if (optsp->do_hex || optsp->do_raw) {
-        if (optsp->do_hex)
+    if (op->do_hex || op->do_raw) {
+        if (op->do_hex)
             dStrHex((const char *)resp, len, 1);
         else
             dStrRaw((const char *)resp, len);
@@ -408,7 +495,7 @@ do_discover(struct smp_target_obj * top, int disc_phy_id,
         if (resp[1] != smp_req[1])
             return -4 - SMP_LIB_CAT_MALFORMED;
         if (resp[2]) {
-            if (optsp->verbose)
+            if (op->verbose)
                 fprintf(stderr, "Discover result: %s\n",
                         smp_get_func_res_str(resp[2], sizeof(b), b));
             return -4 - resp[2];
@@ -426,7 +513,7 @@ do_discover(struct smp_target_obj * top, int disc_phy_id,
         return -4 - SMP_LIB_CAT_MALFORMED;
     }
     if (resp[2]) {
-        if ((optsp->verbose > 0) || (! silence_err_report)) {
+        if ((op->verbose > 0) || (! silence_err_report)) {
             cp = smp_get_func_res_str(resp[2], sizeof(b), b);
             fprintf(stderr, "Discover result: %s\n", cp);
         }
@@ -438,8 +525,8 @@ do_discover(struct smp_target_obj * top, int disc_phy_id,
 /* Note that the inner attributes are output in alphabetical order. */
 /* N.B. This function has not been kept up to date. */
 static int
-do_single_list(const unsigned char * rp, int len, int show_exp_cc,
-               int do_brief)
+print_single_list(const unsigned char * rp, int len, int show_exp_cc,
+                  int do_brief)
 {
     int res, j, sas2;
     unsigned long long ull;
@@ -470,6 +557,7 @@ do_single_list(const unsigned char * rp, int len, int show_exp_cc,
     printf("  att_phy_id=%d\n", rp[32]);
     if (sas2 && (! do_brief)) {
         printf("  att_pow_cap=%d\n", (rp[33] >> 5) & 0x3);
+        printf("  att_pwr_dis_cap=%d\n", !!(rp[34] & 1));
         printf("  att_reason=%d\n", (0xf & rp[12]));
         printf("  att_req_iz=%d\n", !!(0x2 & rp[33]));
     }
@@ -496,6 +584,10 @@ do_single_list(const unsigned char * rp, int len, int show_exp_cc,
             printf("  conn_p_link=%d\n", rp[47]);
             printf("  conn_type=%d\n", (0x7f & rp[45]));
         }
+        if (len > 109) {
+            printf("  dev_slot_num=%d\n", rp[108]);
+            printf("  dev_slot_grp_num=%d\n", rp[109]);
+        }
     }
     if (! do_brief) {
         printf("  hw_max_p_lrate=%d\n", (0xf & rp[41]));
@@ -519,6 +611,10 @@ do_single_list(const unsigned char * rp, int len, int show_exp_cc,
         printf("  pp_timeout=%d\n", (0xf & rp[43]));
         printf("  pr_max_p_lrate=%d\n", ((0xf0 & rp[41]) >> 4));
         printf("  pr_min_p_lrate=%d\n", ((0xf0 & rp[40]) >> 4));
+        if (sas2) {
+            printf("  pwr_dis_ctl_cap=%d\n", (rp[49] & 0x30) >> 4);
+            printf("  pwr_dis_sig=%d\n", (rp[49] & 0xc0) >> 6);
+        }
     }
     if ((! do_brief) && (len > 95))
             printf("  reason=%d\n", (0xf0 & rp[94]) >> 4);
@@ -527,7 +623,7 @@ do_single_list(const unsigned char * rp, int len, int show_exp_cc,
         printf("  req_iz_cbe=%d\n", !! (0x40 & rp[60]));
     }
     printf("  routing_attr=%d\n", rp[44] & 0xf);
-    
+
     for (ull = 0, j = 0; j < 8; ++j) {
         if (j > 0)
             ull <<= 8;
@@ -556,26 +652,67 @@ do_single_list(const unsigned char * rp, int len, int show_exp_cc,
     return 0;
 }
 
-/* Output (multiline) for a single phy. Return 0 on success, positive error
- * number suitable for exit status if problems. */
-static int
-do_single(struct smp_target_obj * top, const struct opts_t * optsp)
+static const char * g_name[] = {"G1", "G2", "G3", "G4"};
+static const char * g_name_long[] =
+        {"G1 (1.5 Gbps)", "G2 (3 Gbps)", "G3 (6 Gbps)", "G4 (12 Gbps)"};
+
+static void
+decode_phy_cap(unsigned int p_cap, const struct opts_t * op)
 {
-    unsigned char rp[SMP_FN_DISCOVER_RESP_LEN];
-    unsigned long long ull;
+    int g14_byte, k, skip, g, prev_nl;
+    const char * cp;
+
+    printf("    Tx SSC type: %d, Requested logical link rate: 0x%x\n",
+           ((p_cap >> 30) & 0x1), ((p_cap >> 24) & 0xf));
+    prev_nl = 1;
+    g14_byte = (p_cap >> 16) & 0xff;
+    for (skip = 0, k = 3; k >= 0; --k) {
+        cp = op->verbose ? g_name_long[3 - k] : g_name[3 - k];
+        g = (g14_byte >> (k * 2)) & 0x3;
+        switch (g) {
+        case 0:
+            ++skip;
+            break;
+        case 1:
+            printf("    %s: with SSC", cp);
+            prev_nl = 0;
+            break;
+        case 2:
+            printf("    %s: without SSC", cp);
+            prev_nl = 0;
+            break;
+        case 3:
+            printf("    %s: with+without SSC", cp);
+            prev_nl = 0;
+            break;
+        default:
+            printf("    %s: g14_byte=0x%x, k=%d", cp, g14_byte, k);
+            prev_nl = 0;
+            break;
+        }
+        if ((2 == k) && (0 == skip)) {
+            printf("\n");
+            skip = 2;
+            prev_nl = 1;
+        }
+        if ((1 == k) && (skip < 2)) {
+            printf("\n");
+            prev_nl = 1;
+        }
+    }
+    if (! prev_nl)
+        printf("\n");
+}
+
+static int
+print_single(const unsigned char * rp, int len, int just1,
+             const struct opts_t * op)
+{
+    int j, sas2, res;
     unsigned int ui;
-    int res, len, j, sas2, ret;
+    unsigned long long ull;
     char b[256];
 
-    /* If do_discover() works, returns response length (less CRC bytes) */
-    len = do_discover(top, optsp->phy_id, rp, sizeof(rp), 0,
-                      optsp);
-    if (len < 0)
-        ret = (len < -2) ? (-4 - len) : len;
-    else
-        ret = 0;
-    if (optsp->do_hex || optsp->do_raw)
-        return ret;
     ull = 0;
     if (len > 23) {
         /* fetch my (expander's) SAS addrss */
@@ -585,35 +722,24 @@ do_single(struct smp_target_obj * top, const struct opts_t * optsp)
             ull |= rp[16 + j];
         }
     }
-    if (optsp->do_my) {
-        printf("0x%llx\n", ull);
-        if ((ull > 0) && (SMP_FRES_PHY_VACANT == ret))
-            return 0;
-        else
-            return ret;
-    }
-    if (ret) {
-        if (SMP_FRES_PHY_VACANT == ret)
-            printf("  phy identifier: %d  inaccessible (phy vacant)\n",
-                   optsp->phy_id);
-        return ret;
-    }
-    if (optsp->do_list)
-        return do_single_list(rp, len, 1, optsp->do_brief);
-    printf("Discover response%s:\n", (optsp->do_brief ? " (brief)" : ""));
+    if (just1)
+        printf("Discover response%s:\n", (op->do_brief ? " (brief)" : ""));
+    else
+        printf("phy identifier: %d\n", rp[9]);
     sas2 = !! (rp[3]);
     res = (rp[4] << 8) + rp[5];
-    if (sas2 || (optsp->verbose > 3)) {
-        if (optsp->verbose || (res > 0))
+    if ((sas2 && (! op->do_brief)) || (op->verbose > 3)) {
+        if (op->verbose || (res > 0))
             printf("  expander change count: %d\n", res);
     }
-    printf("  phy identifier: %d\n", rp[9]);
+    if (just1)
+        printf("  phy identifier: %d\n", rp[9]);
     res = ((0x70 & rp[12]) >> 4);
     if (res < 8)
         printf("  attached device type: %s\n", smp_attached_device_type[res]);
-    if ((optsp->do_brief > 1) && (0 == res))
+    if ((op->do_brief > 1) && (0 == res))
         return 0;
-    if (sas2 || (optsp->verbose > 3))
+    if (sas2 || (op->verbose > 3))
         printf("  attached reason: %s\n",
                smp_get_reason(0xf & rp[12], sizeof(b), b));
 
@@ -622,7 +748,7 @@ do_single(struct smp_target_obj * top, const struct opts_t * optsp)
 
     printf("  attached initiator: ssp=%d stp=%d smp=%d sata_host=%d\n",
            !!(rp[14] & 8), !!(rp[14] & 4), !!(rp[14] & 2), (rp[14] & 1));
-    if (0 == optsp->do_brief) {
+    if (0 == op->do_brief) {
         printf("  attached sata port selector: %d\n",
                !!(rp[15] & 0x80));
         printf("  STP buffer too small: %d\n", !!(rp[15] & 0x10));
@@ -639,8 +765,9 @@ do_single(struct smp_target_obj * top, const struct opts_t * optsp)
     }
     printf("  attached SAS address: 0x%llx\n", ull);
     printf("  attached phy identifier: %d\n", rp[32]);
-    if (0 == optsp->do_brief) {
-        if (sas2 || (optsp->verbose > 3)) {
+    if (0 == op->do_brief) {
+        if (sas2 || (op->verbose > 3)) {
+            printf("  attached persistent capable: %d\n", !!(rp[33] & 0x80));
             printf("  attached power capable: %d\n", ((rp[33] >> 5) & 0x3));
             printf("  attached slumber capable: %d\n", !!(rp[33] & 0x10));
             printf("  attached partial capable: %d\n", !!(rp[33] & 0x8));
@@ -648,6 +775,7 @@ do_single(struct smp_target_obj * top, const struct opts_t * optsp)
                    !!(rp[33] & 4));
             printf("  attached requested inside ZPSDS: %d\n", !!(rp[33] & 2));
             printf("  attached break_reply capable: %d\n", !!(rp[33] & 1));
+            printf("  attached pwr_dis capable: %d\n", !!(rp[34] & 1));
         }
         printf("  programmed minimum physical link rate: %s\n",
                smp_get_plink_rate(((rp[40] >> 4) & 0xf), 1, sizeof(b), b));
@@ -669,7 +797,7 @@ do_single(struct smp_target_obj * top, const struct opts_t * optsp)
     default: snprintf(b, sizeof(b), "reserved [%d]", res); break;
     }
     printf("  routing attribute: %s\n", b);
-    if (optsp->do_brief) {
+    if (op->do_brief) {
         if ((len > 63) && !!(rp[60] & 0x1))
             printf("  zone group: %d\n", rp[63]);
         return 0;
@@ -685,6 +813,8 @@ do_single(struct smp_target_obj * top, const struct opts_t * optsp)
         printf("  sas partial capable: %d\n", !!(rp[48] & 0x4));
         printf("  sata slumber capable: %d\n", !!(rp[48] & 0x2));
         printf("  sata partial capable: %d\n", !!(rp[48] & 0x1));
+        printf("  pwr_dis signal: %d\n", (rp[49] & 0xc0) >> 6);
+        printf("  pwr_dis control capable: %d\n", (rp[49] & 0x30) >> 4);
         printf("  sas slumber enabled: %d\n", !!(rp[49] & 0x8));
         printf("  sas partial enabled: %d\n", !!(rp[49] & 0x4));
         printf("  sata slumber enabled: %d\n", !!(rp[49] & 0x2));
@@ -725,6 +855,8 @@ do_single(struct smp_target_obj * top, const struct opts_t * optsp)
             ui |= rp[76 + j];
         }
         printf("  programmed phy capabilities: 0x%x\n", ui);
+        if (op->do_cap_phy)
+            decode_phy_cap(ui, op);
         ui = 0;
         for (j = 0; j < 4; ++j) {
             if (j > 0)
@@ -732,6 +864,8 @@ do_single(struct smp_target_obj * top, const struct opts_t * optsp)
             ui |= rp[80 + j];
         }
         printf("  current phy capabilities: 0x%x\n", ui);
+        if (op->do_cap_phy)
+            decode_phy_cap(ui, op);
         ui = 0;
         for (j = 0; j < 4; ++j) {
             if (j > 0)
@@ -739,6 +873,8 @@ do_single(struct smp_target_obj * top, const struct opts_t * optsp)
             ui |= rp[84 + j];
         }
         printf("  attached phy capabilities: 0x%x\n", ui);
+        if (op->do_cap_phy)
+            decode_phy_cap(ui, op);
     }
     if (len > 95) {
         printf("  reason: %s\n",
@@ -767,22 +903,69 @@ do_single(struct smp_target_obj * top, const struct opts_t * optsp)
         printf("  shadow zoning enabled: %d\n", !!(rp[104] & 0x1));
         printf("  shadow zone group: %d\n", rp[107]);
     }
-    if (len > 115) {
+    if (len > 109) {
         printf("  device slot number: %d\n", rp[108]);
         printf("  device slot group number: %d\n", rp[109]);
-        printf("  device slot group output connector: %.6s\n", rp + 110);
     }
+    if (len > 115)
+        printf("  device slot group output connector: %.6s\n", rp + 110);
     if (len > 117)
         printf("  STP buffer size: %d\n", (rp[116] << 8) + rp[117]);
     return 0;
 }
 
+
+/* Output (multiline) for a single phy. Return 0 on success, positive error
+ * number suitable for exit status if problems. */
+static int
+do_single(struct smp_target_obj * top, const struct opts_t * op)
+{
+    unsigned char rp[SMP_FN_DISCOVER_RESP_LEN];
+    unsigned long long ull;
+    int j, len, ret;
+
+    /* If do_discover() works, returns response length (less CRC bytes) */
+    len = do_discover(top, op->phy_id, rp, sizeof(rp), 0, op);
+    if (len < 0)
+        ret = (len < -2) ? (-4 - len) : len;
+    else
+        ret = 0;
+    if (op->do_hex || op->do_raw)
+        return ret;
+    ull = 0;
+    if (len > 23) {
+        /* fetch my (expander's) SAS addrss */
+        for (j = 0; j < 8; ++j) {
+            if (j > 0)
+                ull <<= 8;
+            ull |= rp[16 + j];
+        }
+    }
+    if (op->do_my) {
+        printf("0x%llx\n", ull);
+        if ((ull > 0) && (SMP_FRES_PHY_VACANT == ret))
+            return 0;
+        else
+            return ret;
+    }
+    if (ret) {
+        if (SMP_FRES_PHY_VACANT == ret)
+            printf("  phy identifier: %d  inaccessible (phy vacant)\n",
+                   op->phy_id);
+        return ret;
+    }
+    if (op->do_list)
+        return print_single_list(rp, len, 1, op->do_brief);
+    else
+        return print_single(rp, len, 1, op);
+}
+
 #define MAX_PHY_ID 254
 
 /* Calls do_discover() multiple times. Summarizes info into one
  * line per phy. Returns 0 if ok, else function result. */
 static int
-do_multiple(struct smp_target_obj * top, const struct opts_t * optsp)
+do_multiple(struct smp_target_obj * top, const struct opts_t * op)
 {
     unsigned char rp[SMP_FN_DISCOVER_RESP_LEN];
     unsigned long long ull, adn;
@@ -790,12 +973,14 @@ do_multiple(struct smp_target_obj * top, const struct opts_t * optsp)
     int ret, len, k, j, num, off, plus, negot, adt, zg;
     char b[256];
     int first = 1;
+    int checked_rg = 0;
+    int has_t2t = 0;
     const char * cp;
 
     expander_sa = 0;
-    num = optsp->do_num ? (optsp->phy_id + optsp->do_num) : MAX_PHY_ID;
-    for (k = optsp->phy_id; k < num; ++k) {
-        len = do_discover(top, k, rp, sizeof(rp), 1, optsp);
+    num = op->do_num ? (op->phy_id + op->do_num) : MAX_PHY_ID;
+    for (k = op->phy_id; k < num; ++k) {
+        len = do_discover(top, k, rp, sizeof(rp), 1, op);
         if (len < 0)
             ret = (len < -2) ? (-4 - len) : len;
         else
@@ -822,14 +1007,14 @@ do_multiple(struct smp_target_obj * top, const struct opts_t * optsp)
                             "phy_id=%d, was=%llxh, now=%llxh\n", rp[9],
                             expander_sa, ull);
                     expander_sa = ull;
-                } else if (optsp->verbose)
+                } else if (op->verbose)
                     fprintf(stderr, ">> expander's SAS address shown as 0 at "
                             "phy_id=%d\n", rp[9]);
             }
         }
-        if (first && (! optsp->do_raw)) {
+        if (first && (! op->do_raw)) {
             first = 0;
-            if (optsp->sa_given && (optsp->sa != expander_sa))
+            if (op->sa_given && (op->sa != expander_sa))
                 printf("  <<< Warning: reported expander address is not the "
                        "one requested >>>\n");
 #if 0
@@ -838,15 +1023,19 @@ do_multiple(struct smp_target_obj * top, const struct opts_t * optsp)
             printf("Device <%016llx>, expander:\n", expander_sa);
 #endif
         }
-        if (optsp->do_hex || optsp->do_raw)
+        if (op->do_hex || op->do_raw)
             continue;
 
-        if (optsp->do_list) {
-            do_single_list(rp, len, 0, optsp->do_brief);
+        if (op->do_list) {
+            print_single_list(rp, len, 0, op->do_brief);
+            continue;
+        }
+        if (op->multiple > 1) {
+            print_single(rp, len, 0, op);
             continue;
         }
         adt = ((0x70 & rp[12]) >> 4);
-        if ((optsp->do_brief > 1) && (0 == adt))
+        if ((op->do_brief > 1) && (0 == adt))
             continue;
 
         negot = rp[13] & 0xf;
@@ -858,7 +1047,12 @@ do_multiple(struct smp_target_obj * top, const struct opts_t * optsp)
             cp = "S";
             break;
         case 2:
-            cp = "T";
+            if (! checked_rg) {
+                ++checked_rg;
+                has_t2t = has_table2table_routing(top, op);
+            }
+            /* table routing phy when expander does t2t is Universal */
+            cp = has_t2t ? "U" : "T";
             break;
         default:
             cp = "R";
@@ -884,7 +1078,7 @@ do_multiple(struct smp_target_obj * top, const struct opts_t * optsp)
                    cp);
             continue;
         }
-        if ((optsp->do_brief > 0) && (0 == adt))
+        if ((op->do_brief > 0) && (0 == adt))
             continue;
         if (k != rp[9])
             fprintf(stderr, ">> requested phy_id=%d differs from response "
@@ -897,19 +1091,19 @@ do_multiple(struct smp_target_obj * top, const struct opts_t * optsp)
         }
         if ((0 == adt) || (adt > 3)) {
             printf("  phy %3d:%s:attached:[0000000000000000:00]", k, cp);
-            if ((optsp->do_brief > 1) || optsp->do_adn || (len < 64)) {
+            if ((op->do_brief > 1) || op->do_adn || (len < 64)) {
                 printf("\n");
                 continue;
             }
             zg = rp[63];
-            /* (zoning enabled or a zone group > 0) and a zg other than 1 */
-            if (((rp[60] & 0x1) || zg) && (1 != zg))
+            /* zoning_enabled and a zone_group other than 1 */
+            if ((rp[60] & 0x1) && (1 != zg))
                 printf("  ZG:%d\n", zg);
             else
                 printf("\n");
             continue;
         }
-        if (optsp->do_adn && (len > 59)) {
+        if (op->do_adn && (len > 59)) {
             adn = 0;
             for (j = 0; j < 8; ++j) {
                 if (j > 0)
@@ -978,7 +1172,7 @@ do_multiple(struct smp_target_obj * top, const struct opts_t * optsp)
             }
             printf("%s)", b);
         }
-        if ((optsp->do_brief > 1) || optsp->do_adn) {
+        if ((op->do_brief > 1) || op->do_adn) {
             printf("]\n");
             continue;
         } else
@@ -1003,7 +1197,7 @@ do_multiple(struct smp_target_obj * top, const struct opts_t * optsp)
         printf("%s", cp);
         if (len > 63) {
             zg = rp[63];
-            if (((rp[60] & 0x1) || zg) && (1 != zg))
+            if ((rp[60] & 0x1) && (1 != zg))
                 printf("  ZG:%d\n", zg);
             else
                 printf("\n");
@@ -1032,7 +1226,7 @@ main(int argc, char * argv[])
     while (1) {
         int option_index = 0;
 
-        c = getopt_long(argc, argv, "AbhHiI:lmMn:p:rs:SvVz", long_options,
+        c = getopt_long(argc, argv, "AbchHiI:lmMn:p:rs:SvVz", long_options,
                         &option_index);
         if (c == -1)
             break;
@@ -1044,6 +1238,9 @@ main(int argc, char * argv[])
         case 'b':
             ++opts.do_brief;
             break;
+        case 'c':
+            ++opts.do_cap_phy;
+            break;
         case 'h':
         case '?':
             usage();
