@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2006-2011 Douglas Gilbert.
+ * Copyright (c) 2006-2014 Douglas Gilbert.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -48,40 +48,43 @@
  *
  * This utility issues a DISCOVER LIST function and outputs its response.
  *
- * Defined in SAS-2 (most recent draft sas2r16.pdf) and SPL which defines
- * the upper layers of SAS-2.1 . The most recent SPL draft is spl-r07.pdf .
+ * First defined in SAS-2. From and including SAS-2.1 this function is
+ * defined in the SPL series. The most recent SPL-3 draft is spl3r07.pdf .
  */
 
-static char * version_str = "1.27 20111222";    /* spl2r03 */
+static const char * version_str = "1.35 20140526";    /* spl3r07 */
 
 #define MAX_DLIST_SHORT_DESCS 40
 #define MAX_DLIST_LONG_DESCS 8
+#define SMP_FN_REPORT_GENERAL_RESP_LEN 76
 
 static struct option long_options[] = {
-        {"adn", 0, 0, 'A'},
-        {"brief", 0, 0, 'b'},
-        {"descriptor", 1, 0, 'd'},
-        {"filter", 1, 0, 'f'},
-        {"help", 0, 0, 'h'},
-        {"hex", 0, 0, 'H'},
-        {"ignore", 0, 0, 'i'},
-        {"interface", 1, 0, 'I'},
-        {"list", 0, 0, 'l'},    /* placeholder, not implemented */
-        {"num", 1, 0, 'n'},
-        {"one", 0, 0, 'o'},
-        {"phy", 1, 0, 'p'},
-        {"sa", 1, 0, 's'},
-        {"summary", 0, 0, 'S'},
-        {"raw", 0, 0, 'r'},
-        {"verbose", 0, 0, 'v'},
-        {"version", 0, 0, 'V'},
-        {"zpi", 1, 0, 'Z'},
+        {"adn", no_argument, 0, 'A'},
+        {"brief", no_argument, 0, 'b'},
+        {"cap", no_argument, 0, 'c'},
+        {"descriptor", required_argument, 0, 'd'},
+        {"filter", required_argument, 0, 'f'},
+        {"help", no_argument, 0, 'h'},
+        {"hex", no_argument, 0, 'H'},
+        {"ignore", no_argument, 0, 'i'},
+        {"interface", required_argument, 0, 'I'},
+        {"list", no_argument, 0, 'l'},    /* placeholder, not implemented */
+        {"num", required_argument, 0, 'n'},
+        {"one", no_argument, 0, 'o'},
+        {"phy", required_argument, 0, 'p'},
+        {"sa", required_argument, 0, 's'},
+        {"summary", no_argument, 0, 'S'},
+        {"raw", no_argument, 0, 'r'},
+        {"verbose", no_argument, 0, 'v'},
+        {"version", no_argument, 0, 'V'},
+        {"zpi", required_argument, 0, 'Z'},
         {0, 0, 0, 0},
 };
 
 struct opts_t {
     int do_adn;
     int do_brief;
+    int do_cap_phy;
     int desc_type;
     int desc_type_given;
     int filter;
@@ -106,16 +109,15 @@ static void
 usage(void)
 {
     fprintf(stderr, "Usage: "
-          "smp_discover_list  [--adn] [--brief] [--descriptor=TY] "
-          "[--filter=FI]\n"
-          "                          [--help] [--hex] [--ignore] "
-          "[--interface=PARAMS]\n"
-          "                          [--num=NUM] [--one] [--phy=ID] "
-          "[--raw]\n"
-          "                          [--sa=SAS_ADDR] [--summary] "
-          "[--verbose]\n"
-          "                          [--version] [--zpi=FN] "
-          "<smp_device>[,<n>]\n");
+          "smp_discover_list  [--adn] [--brief] [--cap] [--descriptor=TY]\n"
+          "                          [--filter=FI] [--help] [--hex] "
+          "[--ignore]\n"
+          "                          [--interface=PARAMS] [--num=NUM] "
+          "[--one]\n"
+          "                          [--phy=ID] [--raw] [--sa=SAS_ADDR] "
+          "[--summary]\n"
+          "                          [--verbose] [--version] [--zpi=FN]\n"
+          "                          <smp_device>[,<n>]\n");
     fprintf(stderr,
           "  where:\n"
           "    --adn|-A             output attached device name in one "
@@ -123,6 +125,7 @@ usage(void)
           "                         phy mode (i.e. with --one)\n"
           "    --brief|-b           brief: less output, can be used "
           "multiple times\n"
+          "    --cap|-c             decode phy capabilities bits\n"
           "    --descriptor=TY|-d TY    descriptor type:\n"
           "                         0 -> long (as in DISCOVER); 1 -> "
           "short (24 byte)\n"
@@ -131,7 +134,7 @@ usage(void)
           "    --filter=FI|-f FI    phy filter: 0 -> all (def); 1 -> "
           "expander\n"
           "                         attached; 2 -> expander "
-          "or end device\n"
+          "or SAS device\n"
           "    --help|-h            print out usage message\n"
           "    --hex|-H             print response in hexadecimal\n"
           "    --ignore|-i          sets the Ignore Zone Group bit; "
@@ -179,9 +182,91 @@ dStrRaw(const char* str, int len)
         printf("%c", str[k]);
 }
 
-static char * smp_attached_device_type[] = {
+/* Returns 1 if 'Table to Table Supported' bit set on REPORT GENERAL
+ * respone. Returns 0 otherwise. */
+static int
+has_table2table_routing(struct smp_target_obj * top,
+                        const struct opts_t * optsp)
+{
+    unsigned char smp_req[] = {SMP_FRAME_TYPE_REQ, SMP_FN_REPORT_GENERAL,
+                               0, 0, 0, 0, 0, 0};
+    struct smp_req_resp smp_rr;
+    unsigned char rp[SMP_FN_REPORT_GENERAL_RESP_LEN];
+    char b[256];
+    char * cp;
+    int len, res, k, act_resplen;
+
+    memset(rp, 0, sizeof(rp));
+    if (optsp->verbose) {
+        fprintf(stderr, "    Report general request: ");
+        for (k = 0; k < (int)sizeof(smp_req); ++k)
+            fprintf(stderr, "%02x ", smp_req[k]);
+        fprintf(stderr, "\n");
+    }
+    memset(&smp_rr, 0, sizeof(smp_rr));
+    smp_rr.request_len = sizeof(smp_req);
+    smp_rr.request = smp_req;
+    smp_rr.max_response_len = sizeof(rp);
+    smp_rr.response = rp;
+    res = smp_send_req(top, &smp_rr, optsp->verbose);
+
+    if (res) {
+        fprintf(stderr, "RG smp_send_req failed, res=%d\n", res);
+        if (0 == optsp->verbose)
+            fprintf(stderr, "    try adding '-v' option for more debug\n");
+        return 0;
+    }
+    if (smp_rr.transport_err) {
+        fprintf(stderr, "RG smp_send_req transport_error=%d\n",
+                smp_rr.transport_err);
+        return 0;
+    }
+    act_resplen = smp_rr.act_response_len;
+    if ((act_resplen >= 0) && (act_resplen < 4)) {
+        fprintf(stderr, "RG response too short, len=%d\n", act_resplen);
+        return 0;
+    }
+    len = rp[3];
+    if ((0 == len) && (0 == rp[2])) {
+        len = smp_get_func_def_resp_len(rp[1]);
+        if (len < 0) {
+            len = 0;
+            if (optsp->verbose > 1)
+                fprintf(stderr, "unable to determine RG response length\n");
+        }
+    }
+    len = 4 + (len * 4);        /* length in bytes, excluding 4 byte CRC */
+    if ((act_resplen >= 0) && (len > act_resplen)) {
+        if (optsp->verbose)
+            fprintf(stderr, "actual RG response length [%d] less than "
+                    "deduced length [%d]\n", act_resplen, len);
+        len = act_resplen;
+    }
+    /* ignore --hex and --raw */
+    if (SMP_FRAME_TYPE_RESP != rp[0]) {
+        fprintf(stderr, "RG expected SMP frame response type, got=0x%x\n",
+                rp[0]);
+        return 0;
+    }
+    if (rp[1] != smp_req[1]) {
+        fprintf(stderr, "RG Expected function code=0x%x, got=0x%x\n",
+                smp_req[1], rp[1]);
+        return 0;
+    }
+    if (rp[2]) {
+        if (optsp->verbose > 1) {
+            cp = smp_get_func_res_str(rp[2], sizeof(b), b);
+            fprintf(stderr, "Report General result: %s\n", cp);
+        }
+        return 0;
+    }
+    return (len > 10) ? !!(0x80 & rp[10]) : 0;
+}
+
+
+static const char * smp_attached_device_type[] = {
     "no device attached",
-    "end device",
+    "SAS or SATA device",         /* used to be called "end device" */
     "expander device",            /* was 'edge expander' in SAS-1.1 */
     "expander device (fanout)",   /* marked as obsolete in SAS-2.0 */
     "reserved [4]",
@@ -190,7 +275,7 @@ static char * smp_attached_device_type[] = {
     "reserved [7]",
 };
 
-static char * smp_short_attached_device_type[] = {
+static const char * smp_short_attached_device_type[] = {
     "",         /* was "no " */
     "",         /* was "end" */
     "exp",
@@ -419,7 +504,7 @@ do_discover_list(struct smp_target_obj * top, int sphy_id,
         if (op->verbose)
             fprintf(stderr, "actual response length [%d] less than "
                     "deduced length [%d]\n", act_resplen, len);
-        len = act_resplen; 
+        len = act_resplen;
     }
     if (op->do_hex || op->do_raw) {
         if (op->do_hex)
@@ -458,6 +543,58 @@ do_discover_list(struct smp_target_obj * top, int sphy_id,
     return 0;
 }
 
+static const char * g_name[] = {"G1", "G2", "G3", "G4"};
+static const char * g_name_long[] =
+        {"G1 (1.5 Gbps)", "G2 (3 Gbps)", "G3 (6 Gbps)", "G4 (12 Gbps)"};
+
+static void
+decode_phy_cap(unsigned int p_cap, const struct opts_t * op)
+{
+    int g14_byte, k, skip, g, prev_nl;
+    const char * cp;
+
+    printf("    Tx SSC type: %d, Requested logical link rate: 0x%x\n",
+           ((p_cap >> 30) & 0x1), ((p_cap >> 24) & 0xf));
+    prev_nl = 1;
+    g14_byte = (p_cap >> 16) & 0xff;
+    for (skip = 0, k = 3; k >= 0; --k) {
+        cp = op->verbose ? g_name_long[3 - k] : g_name[3 - k];
+        g = (g14_byte >> (k * 2)) & 0x3;
+        switch (g) {
+        case 0:
+            ++skip;
+            break;
+        case 1:
+            printf("    %s: with SSC", cp);
+            prev_nl = 0;
+            break;
+        case 2:
+            printf("    %s: without SSC", cp);
+            prev_nl = 0;
+            break;
+        case 3:
+            printf("    %s: with+without SSC", cp);
+            prev_nl = 0;
+            break;
+        default:
+            printf("    %s: g14_byte=0x%x, k=%d", cp, g14_byte, k);
+            prev_nl = 0;
+            break;
+        }
+        if ((2 == k) && (0 == skip)) {
+            printf("\n");
+            skip = 2;
+            prev_nl = 1;
+        }
+        if ((1 == k) && (skip < 2)) {
+            printf("\n");
+            prev_nl = 1;
+        }
+    }
+    if (! prev_nl)
+        printf("\n");
+}
+
 /* long format: as described in (full, single) DISCOVER response
  * Returns 0 for okay, else -1 . */
 static int
@@ -523,12 +660,14 @@ decode_desc0_multiline(const unsigned char * resp, int offset,
     printf("  attached SAS address: 0x%llx\n", ull);
     printf("  attached phy identifier: %d\n", rp[32]);
     if (0 == op->do_brief) {
+        printf("  attached persistent capable: %d\n", !!(rp[33] & 0x80));
         printf("  attached power capable: %d\n", ((rp[33] >> 5) & 0x3));
         printf("  attached slumber capable: %d\n", !!(rp[33] & 0x10));
         printf("  attached partial capable: %d\n", !!(rp[33] & 0x8));
         printf("  attached inside ZPSDS persistent: %d\n", !!(rp[33] & 4));
         printf("  attached requested inside ZPSDS: %d\n", !!(rp[33] & 2));
         printf("  attached break_reply capable: %d\n", !!(rp[33] & 1));
+        printf("  attached pwr_dis capable: %d\n", !!(rp[34] & 1));
         printf("  programmed minimum physical link rate: %s\n",
                smp_get_plink_rate(((rp[40] >> 4) & 0xf), 1,
                                   sizeof(b), b));
@@ -567,6 +706,8 @@ decode_desc0_multiline(const unsigned char * resp, int offset,
     printf("  sas partial capable: %d\n", !!(rp[48] & 0x4));
     printf("  sata slumber capable: %d\n", !!(rp[48] & 0x2));
     printf("  sata partial capable: %d\n", !!(rp[48] & 0x1));
+    printf("  pwr_dis signal: %d\n", (rp[49] & 0xc0) >> 6);
+    printf("  pwr_dis control capable: %d\n", (rp[49] & 0x30) >> 4);
     printf("  sas slumber enabled: %d\n", !!(rp[49] & 0x8));
     printf("  sas partial enabled: %d\n", !!(rp[49] & 0x4));
     printf("  sata slumber enabled: %d\n", !!(rp[49] & 0x2));
@@ -591,7 +732,7 @@ decode_desc0_multiline(const unsigned char * resp, int offset,
         if (len < 76)
             return 0;
         printf("  self-configuration status: %d\n", rp[64]);
-        printf("  self-configuration level completed: %d\n", rp[65]);
+        printf("  self-configuration levels completed: %d\n", rp[65]);
         ull = 0;
         for (j = 0; j < 8; ++j) {
             if (j > 0)
@@ -606,6 +747,8 @@ decode_desc0_multiline(const unsigned char * resp, int offset,
             ui |= rp[76 + j];
         }
         printf("  programmed phy capabilities: 0x%x\n", ui);
+        if (op->do_cap_phy)
+            decode_phy_cap(ui, op);
         ui = 0;
         for (j = 0; j < 4; ++j) {
             if (j > 0)
@@ -613,6 +756,8 @@ decode_desc0_multiline(const unsigned char * resp, int offset,
             ui |= rp[80 + j];
         }
         printf("  current phy capabilities: 0x%x\n", ui);
+        if (op->do_cap_phy)
+            decode_phy_cap(ui, op);
         ui = 0;
         for (j = 0; j < 4; ++j) {
             if (j > 0)
@@ -620,6 +765,8 @@ decode_desc0_multiline(const unsigned char * resp, int offset,
             ui |= rp[84 + j];
         }
         printf("  attached phy capabilities: 0x%x\n", ui);
+        if (op->do_cap_phy)
+            decode_phy_cap(ui, op);
     }
     if (len > 95) {
         printf("  reason: %s\n",
@@ -648,13 +795,13 @@ decode_desc0_multiline(const unsigned char * resp, int offset,
         printf("  shadow zoning enabled: %d\n", !!(rp[104] & 0x1));
         printf("  shadow zone group: %d\n", rp[107]);
     }
-    if (len > 115) {
+    if (len > 109) {
         printf("  device slot number: %d\n", rp[108]);
         printf("  device slot group number: %d\n", rp[109]);
-        printf("  device slot group output connector: %.6s\n", rp + 110);
-
     }
-    if (len > 117) 
+    if (len > 115)
+        printf("  device slot group output connector: %.6s\n", rp + 110);
+    if (len > 117)
         printf("  STP buffer size: %d\n", (rp[116] << 8) + rp[117]);
     return 0;
 }
@@ -663,7 +810,7 @@ decode_desc0_multiline(const unsigned char * resp, int offset,
  * Returns 0 for okay, else -1 . */
 static int
 decode_desc1_multiline(const unsigned char * resp, int offset,
-                       int z_supported, struct opts_t * op)
+                       int z_enabled, struct opts_t * op)
 {
     const unsigned char *rp;
     unsigned long long ull;
@@ -721,7 +868,7 @@ decode_desc1_multiline(const unsigned char * resp, int offset,
     }
     printf("  routing attribute: %s\n", b);
     if (op->do_brief) {
-        if (z_supported)
+        if (z_enabled)
             printf("  zone group: %d\n", rp[8]);
         return 0;
     }
@@ -739,11 +886,11 @@ decode_desc1_multiline(const unsigned char * resp, int offset,
 }
 
 /* Decodes either descriptor type into one line output. This is a
- * "per phy" function. Returns 0 for ok, 1 for ok plus seen ZG other
- * than ZG:1, else -1 . */
+ * "per phy" function. Returns 0 for ok, 1 for ok plus zoning enabled and
+  * seen ZG other than 1, else -1 (for problem) . */
 static int
 decode_1line(const unsigned char * resp, int offset, int desc,
-             int z_supported, struct opts_t * op)
+             int z_enabled, int has_t2t, struct opts_t * op)
 {
     const unsigned char *rp;
     unsigned long long ull, adn;
@@ -817,8 +964,8 @@ decode_1line(const unsigned char * resp, int offset, int desc,
     case 1:
         cp = "S";
         break;
-    case 2:
-        cp = "T";
+    case 2:     /* table routing phy when expander does t2t is Universal */
+        cp = has_t2t ? "U" : "T";
         break;
     default:
         cp = "R";
@@ -854,7 +1001,7 @@ decode_1line(const unsigned char * resp, int offset, int desc,
             printf("\n");
             return 0;
         }
-        if (z_supported && (1 != z_group)) {
+        if (z_enabled && (1 != z_group)) {
             ++zg_not1;
             printf("  ZG:%d\n", z_group);
         } else
@@ -950,7 +1097,7 @@ decode_1line(const unsigned char * resp, int offset, int desc,
             break;
         }
         printf("%s", cp);
-        if (z_supported && (1 != z_group)) {
+        if (z_enabled && (1 != z_group)) {
             ++zg_not1;
             printf("  ZG:%d", z_group);
         }
@@ -1021,9 +1168,10 @@ main(int argc, char * argv[])
     struct smp_target_obj tobj;
     int subvalue = 0;
     char * cp;
-    int z_supported = 0;
     int z_enabled = 0;
     int zg_not1 = 0;
+    int checked_rg = 0;
+    int has_t2t = 0;
     int ret = 0;
     struct opts_t opts;
 
@@ -1032,8 +1180,8 @@ main(int argc, char * argv[])
     while (1) {
         int option_index = 0;
 
-        c = getopt_long(argc, argv, "Abd:f:hHiI:ln:op:rs:SvVZ:", long_options,
-                        &option_index);
+        c = getopt_long(argc, argv, "Abcd:f:hHiI:ln:op:rs:SvVZ:",
+                        long_options, &option_index);
         if (c == -1)
             break;
 
@@ -1044,6 +1192,9 @@ main(int argc, char * argv[])
         case 'b':
             ++opts.do_brief;
             break;
+        case 'c':
+            ++opts.do_cap_phy;
+            break;
         case 'd':
            opts.desc_type = smp_get_num(optarg);
            if ((opts.desc_type < 0) || (opts.desc_type > 15)) {
@@ -1189,9 +1340,10 @@ main(int argc, char * argv[])
         (NULL == opts.zpi_fn))
         ++opts.do_summary;
     if (opts.zpi_fn) {
-        if (opts.do_summary || opts.desc_type_given || opts.filter || opts.do_adn) {
-            fprintf(stderr, "--zpi=FN clashes with --summary, --adn, --filter and "
-                    "--descriptor=TY options\n");
+        if (opts.do_summary || opts.desc_type_given || opts.filter ||
+            opts.do_adn) {
+            fprintf(stderr, "--zpi=FN clashes with --summary, --adn, "
+                    "--filter and --descriptor=TY options\n");
             return SMP_LIB_SYNTAX_ERROR;
         }
         if (! opts.num_given)
@@ -1264,7 +1416,6 @@ main(int argc, char * argv[])
         if ((0 == j) && ((! opts.do_1line) || opts.zpi_fn))
             output_header_info(resp, &opts);
         hdr_ecc = (resp[4] << 8) + resp[5];
-        z_supported = !!(resp[16] & 0x80);
         z_enabled = !!(resp[16] & 0x40);
         resp_filter = resp[10] & 0xf;
         if (opts.filter != resp_filter)
@@ -1287,8 +1438,12 @@ main(int argc, char * argv[])
         for (k = 0, err = 0; k < num_desc; ++k) {
             off = 48 + (k * desc_len);
             if (opts.do_1line) {
-                res = decode_1line(resp, off, resp_desc_type, z_supported,
-                                   &opts);
+                if (! checked_rg) {
+                    ++checked_rg;
+                    has_t2t = has_table2table_routing(&tobj, &opts);
+                }
+                res = decode_1line(resp, off, resp_desc_type, z_enabled,
+                                   has_t2t, &opts);
                 if (res < 0)
                     ++err;
                 else if (res > 0)
@@ -1306,7 +1461,7 @@ main(int argc, char * argv[])
                     adt = (resp[off + 2] >> 4) & 7;
                     if ((0 == opts.do_brief) || adt || fresult) {
                         printf("descriptor %d:\n", j + k);
-                        if (decode_desc1_multiline(resp, off, z_supported,
+                        if (decode_desc1_multiline(resp, off, z_enabled,
                                                    &opts))
                             ++err;
                     }
@@ -1324,7 +1479,7 @@ main(int argc, char * argv[])
     }
     if ((zg_not1) && (0 == opts.do_brief) && (NULL == opts.zpi_fn))
         printf("Zoning %sabled\n", z_enabled ? "en" : "dis");
-    
+
 err_out:
     if (opts.zpi_filep && (stdout != opts.zpi_filep))
         fclose(opts.zpi_filep);
