@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2011 Douglas Gilbert.
+ * Copyright (c) 2011-2014 Douglas Gilbert.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -50,7 +50,7 @@
  * response.
  */
 
-static char * version_str = "1.07 20111222";
+static const char * version_str = "1.08 20130604";
 
 #define SMP_FN_REPORT_PHY_EVENT_LIST_RESP_LEN (1020 + 4 + 4)
 
@@ -133,9 +133,10 @@ usage(void)
           "[--help]\n"
           "                              [--hex] [--index=IN] "
           "[--interface=PARAMS]\n"
-          "                              [--long] [--raw] [--sa=SAS_ADDR] "
-          "[--verbose]\n"
-          "                              [--version] SMP_DEVICE[,N]\n"
+          "                              [--long] [--nonz] [--raw] "
+          "[--sa=SAS_ADDR]\n"
+          "                              [--verbose] [--version] "
+          "SMP_DEVICE[,N]\n"
           "  where:\n"
           "    --desc|-d            show descriptor number in output\n"
           "    --enumerate|-e       enumerate phy event source names, "
@@ -508,7 +509,7 @@ main(int argc, char * argv[])
         if (verbose)
             fprintf(stderr, "actual response length [%d] less than deduced "
                     "length [%d]\n", act_resplen, len);
-        len = act_resplen; 
+        len = act_resplen;
     }
     if (do_hex || do_raw) {
         if (do_hex)
